//
// legocore.c 
// 7-2-2003 Ted Rossin
// 7-2-2003
//

#include <pic.h>
#include "legocore.h"

static unsigned char Tick50us;
static unsigned int Tick1ms;

static unsigned char PwmValue[4];
static unsigned char AnalogValue[8];
static unsigned char PwmCounter;
static unsigned char ADCState;
static unsigned char ADCChannel;

/*************************************************************************
**									                                    **
** Interrupt handler						    	                    **
**      Timer interrupt happens 250/5000000 seconds = 50us		        **
**									                                    **
*************************************************************************/

void InitTimers(void)
{
    OPTION = 0xd8; // 1/1 prescaler
	TMR0 = -250;
	T0CS = 0;		// Timer increments on instruction clock
	T0IE = 1;		// Enable interrupt on TMR0 overflow
	GIE = 1;		// Global interrupt enable
	Tick50us = 0;
	                // Timer 1 is used for the wait functions (not really)
	T1CON = 0x01;
	TMR1L = 0;
	TMR1H = 0;
}

void Waitms(unsigned int Num)
{
    Tick1ms = 0;
    while(Tick1ms<Num);
}

void interrupt isr(void) // function name is a don't care
{
		// Update Timer
	TMR0 -= 250; 		// reload the timer - 50uS per interrupt
	T0IF = 0;			// clear the interrupt flag
	Tick50us++;
	if(Tick50us==20){ 
		Tick1ms++;
		Tick50us = 0;
	}

        // Update PWM outputs

    if(PwmCounter==0) PORTB |= 0x0f;  // Turn on all pwm outputs
    if(PwmCounter >= PwmValue[0]) RB0 = 0;
    if(PwmCounter >= PwmValue[1]) RB1 = 0;
    if(PwmCounter >= PwmValue[2]) RB2 = 0;
    if(PwmCounter >= PwmValue[3]) RB3 = 0;
    PwmCounter += 4;

        // ADC sampling state machine

	if(ADCState & 0x01){  // Get result of conversion
		AnalogValue[ADCChannel++] = ADRESH;
		ADCChannel &= 0x07;
			// Update Analog Mux to new channel
		CHS2 = 0;  CHS1 = 0;  CHS0 = 0;
		if(ADCChannel & 4) CHS2 = 1;
		if(ADCChannel & 2) CHS1 = 1; 
		if(ADCChannel & 1) CHS0 = 1;
		ADCState = 0;	
	}
	else{  // Start Conversion
		ADGO = 1;	// Start A/D conversion
		ADCState = 1;	// Next state is state 1
	}
}

void LegoInit(void)
{
    TRISA = 0x2f;   // Port A[3:0],A[5] are input for A/D	
    TRISB = 0x00;   // PORT B[7:0] is Output
    TRISC = 0x00;   // PORT C[7:0] is output
    TRISD = 0x00;   // PORT D[7:0] is output
    TRISE = 0x07;   // PORT E[2:0] is input for A/D

	ADCON1 = 0x00;	// (left justified data) 8 or 5 A/D channels
	ADCON0 = 0x81;  // FOSC/32, channel=0, ADON=1

    PwmValue[0] = 0;
    PwmValue[1] = 0;
    PwmValue[2] = 0;
    PwmValue[3] = 0;

    ADCState = 0;
    ADCChannel = 0;

    InitTimers();
}


void SetOutputValue(unsigned char flags,unsigned char value)
{
    if(flags & 1) PwmValue[0] = value;
    if(flags & 2) PwmValue[1] = value;
    if(flags & 4) PwmValue[2] = value;
    if(flags & 8) PwmValue[3] = value;
}

unsigned char GetInput(unsigned char flags)
{
    if(flags & 0x01) return(AnalogValue[0]);
    if(flags & 0x02) return(AnalogValue[1]);
    if(flags & 0x04) return(AnalogValue[2]);
    if(flags & 0x08) return(AnalogValue[3]);
    if(flags & 0x10) return(AnalogValue[4]);
    if(flags & 0x20) return(AnalogValue[5]);
    if(flags & 0x40) return(AnalogValue[6]);
    if(flags & 0x80) return(AnalogValue[7]);
    return(0);
}

/*************************************************************************
**									                                    **
** SetBrake: Sets channels passed in flags to Brake (both outputs to 	**
**     	     zero.							                            **
**									                                    **
*************************************************************************/

void SetBrake(unsigned char flags)
{
    if(flags & 0x01){
        RB4 = 0;  // Clear Direction
        RB5 = 0;
        PwmValue[0] = 0xff;
    }
    if(flags & 0x02){
        RB6 = 0;  // Clear Direction
        RB7 = 0;
        PwmValue[1] = 0xff;
    }
    if(flags & 0x04){
        RC2 = 0;  // Clear Direction
        RC3 = 0;
        PwmValue[2] = 0xff;
    }
    if(flags & 0x08){
        RC4 = 0;  // Clear Direction
        RC5 = 0;
        PwmValue[3] = 0xff;
    }
}

/*************************************************************************
**								    	                                **
** SetFloat: Sets channels passed in flags to float.			        **
**									                                    **
*************************************************************************/
void SetFloat(unsigned char flags)
{
    if(flags & 0x01) PwmValue[0] = 0;
    if(flags & 0x02) PwmValue[1] = 0;
    if(flags & 0x04) PwmValue[2] = 0;
    if(flags & 0x08) PwmValue[3] = 0;
}

/*************************************************************************
**									                                    **
** SetForward: Sets channels passed in flags to forward.		        **
**									                                    **
*************************************************************************/
void SetForward(unsigned char flags)
{
    if(flags & 0x01){
        RB4 = 1; 
        RB5 = 0;
    }
    if(flags & 0x02){
        RB6 = 1;
        RB7 = 0;
    }
    if(flags & 0x04){
        RC2 = 1;
        RC3 = 0;
    }
    if(flags & 0x08){
        RC4 = 1;
        RC5 = 0;
    }
}

/*************************************************************************
**									                                    **
** SetReverse: Sets channels passed in flags to Reverse.    	        **
**									                                    **
*************************************************************************/
void SetReverse(unsigned char flags)
{
    if(flags & 0x01){
        RB4 = 0; 
        RB5 = 1;
    }
    if(flags & 0x02){
        RB6 = 0;
        RB7 = 1;
    }
    if(flags & 0x04){
        RC2 = 0;
        RC3 = 1;
    }
    if(flags & 0x08){
        RC4 = 0;
        RC5 = 1;
    }
}
